/*************************************************************************
 *
 * mod_mixer_save.c -- Stores the settings of the volume, bass, treble and balance
 *            in a file and restores them on startup
 *
 * Copyright (C) by Marco Glatz <m.glatz@fanatic-minds.de>
 *
 */


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#include "tools.h"
#include "log.h"
#include "mod.h"
#include "config.h"
#include "mod_mixer_save.h"


/*************************************************************************
 * MODULE INFO
 */
mod_t mod_mixer_save = {
  mod_mixer_save_deinit,  // deinit
  mod_mixer_save_reload,  // reload
  0,      // watchfd
  NULL,      // poll
  NULL,      // update
  mod_mixer_save_message,  // omessage
  NULL,      // SIGCHLD handler
};



/*************************************************************************
 * GLOBALS
 */
extern int daemonize;     // set by irmp3.c if we are a daemon

char  *MOD_MIXER_SAVE_VOLUME_FILE  = NULL;
char  *MOD_MIXER_SAVE_BALANCE_FILE = NULL;
char  *MOD_MIXER_SAVE_TREBLE_FILE  = NULL;
char  *MOD_MIXER_SAVE_BASS_FILE    = NULL;

int MOD_MIXER_SAVE_STORE_VOLUME  = 0;
int MOD_MIXER_SAVE_STORE_BALANCE = 0;
int MOD_MIXER_SAVE_STORE_TREBLE  = 0;
int MOD_MIXER_SAVE_STORE_BASS    = 0;

int MOD_MIXER_SAVE_INIT = 1;


/*************************************************************************
 * RECEIVE MESSAGE
 */
void mod_mixer_save_message (int msgtype, char *msg)
{
  char *c1, *c2;

  log_printf(LOG_NOISYDEBUG, "mod_mixer_save_message(): got message type %d: '%s'\n", msgtype, msg);

  // handle mixer messages
  if (msgtype == MSGTYPE_MIXER) {
    c1 = msg ? strtok(msg, " \t") : NULL;
    c2 = c1 ? strtok(NULL, " \t") : NULL;

    /*
    * volume
    */
    if (c1 && !strcasecmp(c1, "volume")) {
      log_printf(LOG_DEBUG, "mod_mixer_save_message(): got volume setting '%s'\n", c2);

      mod_mixer_save_store("volume", c2 ? atoi(c2) : 0);

    /*
    * balance
    */
    } else if (c1 && !strcasecmp(c1, "balance")) {
      log_printf(LOG_DEBUG, "mod_mixer_save_message(): got balance setting '%s'\n", c2);

      mod_mixer_save_store("balance", c2 ? atoi(c2) : 0);

    /*
    * bass
    */
    } else if (c1 && !strcasecmp(c1, "bass")) {
      log_printf(LOG_DEBUG, "mod_mixer_save_message(): got bass setting '%s'\n", c2);

      mod_mixer_save_store("bass", c2 ? atoi(c2) : 0);

    /*
    * treble
    */
    } else if (c1 && !strcasecmp(c1, "treble")) {
      log_printf(LOG_DEBUG, "mod_mixer_save_message(): got treble setting '%s'\n", c2);

      mod_mixer_save_store("treble", c2 ? atoi(c2) : 0);


  // handle input messages
  } else if (msgtype == MSGTYPE_INPUT) {

    /*
    * reset
    */
    } if (!strcasecmp(msg, "mixer default")) {
      mod_mixer_save_reset();
    }
  }
}


/*************************************************************************
 * READ SETTINGS
 */
int mod_mixer_save_read (char *what)
{
  int val = 0;
    int fd  = 0;

    char valc[3];

  int default_v  = 0;
    char *file     = NULL;
    char *config_v = NULL;

  // volume
    if(what == "volume") {
        file      = MOD_MIXER_SAVE_VOLUME_FILE;
        config_v  = "mixer_volume";
        default_v = -1;

  // bass
    } else if(what == "bass") {
        file      = MOD_MIXER_SAVE_BASS_FILE;
        config_v  = "mixer_bass";
        default_v = -1;

  // treble
    } else if(what == "treble") {
        file      = MOD_MIXER_SAVE_TREBLE_FILE;
        config_v  = "mixer_treble";
        default_v = -1;

  // balance
    } else {
        file      = MOD_MIXER_SAVE_BALANCE_FILE;
        config_v  = "mixer_balance";
        default_v = 0;
    }

  // reading failed, use settings from config-file
      if ((fd = open(file, O_RDONLY)) == -1) {
          val = config_getnum(config_v, default_v);
          log_printf(LOG_DEBUG, "mod_mixer_save_read(): unable to read '%s' from file '%s'.\n", what, file);

  // read setting from file
      } else {
          readline(fd, valc, sizeof(valc));

    // nothing stored, use settings from config-file
          if(valc == "") {
            val = config_getnum(config_v, default_v);

    // get value
          } else {
            val = atoi(valc);
          }

        log_printf(LOG_DEBUG, "mod_mixer_read(): reading '%s' from file '%s', value = '%d'\n", what, file, val);
      }

    close(fd);

    return val;
}


/*************************************************************************
 * STORE SETTINGS
 */
void mod_mixer_save_store (char *what, int value)
{
    int fd      = 0;
  int enabled = 0;
    char *file  = NULL;

  // volume
    if(what == "volume") {
        file = MOD_MIXER_SAVE_VOLUME_FILE;

    if(MOD_MIXER_SAVE_STORE_VOLUME == 1) {
      enabled = 1;
    }

  // bass
    } else if(what == "bass") {
        file = MOD_MIXER_SAVE_BASS_FILE;

    if(MOD_MIXER_SAVE_STORE_BASS == 1) {
      enabled = 1;
    }

  // treble
    } else if(what == "treble") {
        file = MOD_MIXER_SAVE_TREBLE_FILE;

    if(MOD_MIXER_SAVE_STORE_TREBLE == 1) {
      enabled = 1;
    }

  // balance
    } else {
        file = MOD_MIXER_SAVE_BALANCE_FILE;

    if(MOD_MIXER_SAVE_STORE_BALANCE == 1) {
      enabled = 1;
    }
    }

  // we are not in init-mode and storing is enabled
  if(MOD_MIXER_SAVE_INIT == 0 && enabled == 1) {
        fd = open(file, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);

    // reading/creating failed
        if(fd < 0) {
            log_printf(LOG_DEBUG, "mod_mixer_save_store(): unable to save '%s' to file '%s'\n", what, file);
        } else {
            sendtext(fd, "%d", value);
            log_printf(LOG_DEBUG, "mod_mixer_save_store(): writing '%s' to file '%s', value = '%d'\n", what, file, value);
        }

      close(fd);
  }
}


/*************************************************************************
 * RESET TO DEFAULT SETTINGS
 */
void mod_mixer_save_reset (void)
{
  log_printf(LOG_DEBUG, "mod_mixer_save: resetting to default-values.\n");

    // volume
  if(MOD_MIXER_SAVE_STORE_VOLUME == 1) {
     mod_mixer_save_store("volume", config_getnum("mixer_volume", -1));
  }

  // balance
  if(MOD_MIXER_SAVE_STORE_BALANCE == 1) {
     mod_mixer_save_store("balance", config_getnum("mixer_balance", 0));
  }

    // bass
    if(MOD_MIXER_SAVE_STORE_BASS == 1) {
    mod_mixer_save_store("bass", config_getnum("mixer_bass", -1));
  }

    // treble
    if(MOD_MIXER_SAVE_STORE_TREBLE == 1) {
    mod_mixer_save_store("treble", config_getnum("mixer_treble", -1));
  }
}


/*************************************************************************
 * SETUP
 */
void mod_mixer_save_setup (void)
{
  int val = 0;


  /*
  * volume
  */
  MOD_MIXER_SAVE_VOLUME_FILE = config_getstr("mixer_save_volume", NULL);

  if(MOD_MIXER_SAVE_VOLUME_FILE != NULL) {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing volume = '%s'\n", MOD_MIXER_SAVE_VOLUME_FILE);

    // storing enabled
    MOD_MIXER_SAVE_STORE_VOLUME = 1;

    val = mod_mixer_save_read("volume");

    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): setting volume to '%d'\n", val);
    mod_sendmsgf(MSGTYPE_INPUT, "mixer volume %d", val);

  } else {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing volume not found. skipping.\n");
  }


  /*
  * balance
  */
  MOD_MIXER_SAVE_BALANCE_FILE = config_getstr("mixer_save_balance", NULL);

  if(MOD_MIXER_SAVE_BALANCE_FILE != NULL) {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing balance = '%s'\n", MOD_MIXER_SAVE_BALANCE_FILE);

    // storing enabled
    MOD_MIXER_SAVE_STORE_BALANCE = 1;

    val = mod_mixer_save_read("balance");

    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): setting balance to '%d'\n", val);
    mod_sendmsgf(MSGTYPE_INPUT, "mixer balance %d", val);

  } else {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing balance not found. skipping.\n");
  }


  /*
  * treble
  */
  MOD_MIXER_SAVE_TREBLE_FILE = config_getstr("mixer_save_treble", NULL);

  if(MOD_MIXER_SAVE_TREBLE_FILE != NULL) {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing treble = '%s'\n", MOD_MIXER_SAVE_TREBLE_FILE);

    // storing enabled
    MOD_MIXER_SAVE_STORE_TREBLE = 1;

    val = mod_mixer_save_read("treble");

    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): setting treble to '%d'\n", val);
    mod_sendmsgf(MSGTYPE_INPUT, "mixer treble %d", val);

  } else {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing treble not found. skipping.\n");
  }


  /*
  * bass
  */
  MOD_MIXER_SAVE_BASS_FILE = config_getstr("mixer_save_bass", NULL);

  if(MOD_MIXER_SAVE_BASS_FILE != NULL) {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing bass = '%s'\n", MOD_MIXER_SAVE_VOLUME_FILE);

    // storing enabled
    MOD_MIXER_SAVE_STORE_BASS = 1;

    val = mod_mixer_save_read("bass");

    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): setting bass to '%d'\n", val);
    mod_sendmsgf(MSGTYPE_INPUT, "mixer bass %d", val);

  } else {
    log_printf(LOG_NOISYDEBUG, "mod_mixer_save_init(): file for storing bass not found. skipping.\n");
  }
}


/*************************************************************************
 * MODULE INIT FUNCTION
 */
char *mod_mixer_save_init (void)
{
  log_printf(LOG_DEBUG, "mod_mixer_save_init(): initializing\n");

  // register
  mod_register(&mod_mixer_save);

  // read settings
  mod_mixer_save_setup();

  // init done
  MOD_MIXER_SAVE_INIT = 0;

  return NULL;
}


/*************************************************************************
 * MODULE DEINIT FUNCTION
 */
void mod_mixer_save_deinit (void)
{
  MOD_MIXER_SAVE_VOLUME_FILE  = NULL;
  MOD_MIXER_SAVE_BALANCE_FILE = NULL;
  MOD_MIXER_SAVE_TREBLE_FILE  = NULL;
  MOD_MIXER_SAVE_BASS_FILE    = NULL;

  MOD_MIXER_SAVE_STORE_VOLUME  = 0;
  MOD_MIXER_SAVE_STORE_BALANCE = 0;
  MOD_MIXER_SAVE_STORE_TREBLE  = 0;
  MOD_MIXER_SAVE_STORE_BASS    = 0;

  MOD_MIXER_SAVE_INIT = 1;

  log_printf(LOG_DEBUG, "mod_mixer_save_deinit(): deinitialized\n");
}


/*************************************************************************
 * MODULE RELOAD FUNCTION
 */
char *mod_mixer_save_reload (void)
{
  log_printf(LOG_DEBUG, "mod_mixer_save_reload(): reloading\n");
  mod_mixer_save_setup();

  return NULL;
}


/*************************************************************************
 * EOF
 */


